local defaultReplaces = {
	["{stern}"] = "{rt1}",
	["{kreis}"] = "{rt2}",
	["{diamant}"] = "{rt3}",
	["{dreieck}"] = "{rt4}",
	["{mond}"] = "{rt5}",
	["{quadrat}"] = "{rt6}",
	["{kreuz}"] = "{rt7}",
	["{totensch\195\164del}"] = "{rt8}",
}

local defaultChats = {
	"CHAT_MSG_AFK",
	"CHAT_MSG_DND",
	"CHAT_MSG_EMOTE",
	"CHAT_MSG_OFFICER",
	"CHAT_MSG_BATTLEGROUND_LEADER",
	"CHAT_MSG_WHISPER",
	"CHAT_MSG_GUILD",
	"CHAT_MSG_CHANNEL",
	"CHAT_MSG_YELL",
	"CHAT_MSG_SAY",
	"CHAT_MSG_BATTLEGROUND",
	"CHAT_MSG_PARTY",
	"CHAT_MSG_PARTY_LEADER",
	"CHAT_MSG_RAID",
	"CHAT_MSG_RAID_LEADER",
	"CHAT_MSG_RAID_WARNING",
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_WHISPER_INFORM",
	"CHAT_MSG_MONSTER_YELL",
	"CHAT_MSG_TEXT_EMOTE",
}

local defaultIgnores = {}

local defaultExactIgnores = {}

local defaultDelim = ":"

local defaultStats = {
	["p"] = 0, -- passed unaltered
	["f"] = 0, -- filtered
	["e"] = 0, -- filtered exact
	["r"] = 0, -- passed replaced
}

local r = {}

CF_CHATS = CF_CHATS or defaultChats
CF_WORDS_REPLACE = CF_WORDS_REPLACE or defaultReplaces
CF_WORDS_IGNORE = CF_WORDS_IGNORE or defaultIgnores
CF_WORDS_EXACT_IGNORE = CF_WORDS_EXACT_IGNORE or defaultExactIgnores
CF_DELIM = CF_DELIM or defaultDelim
CF_STATS = CF_STATS or defaultStats

local function generateList()
	r = {}
	for k, v in pairs(CF_WORDS_REPLACE) do
		r[k:gsub("%a",function(a) return "["..a:upper()..a:lower().."]" end)] = v
	end
end

local function filter(self, event, msg, ...)
	local newMsg = msg
	for k, v in pairs(CF_WORDS_IGNORE) do
		if msg:find(k) then
			if (type(v) == "boolean" and v == true) or (type(v) == "string" and self["name"] == v) then
				CF_STATS["f"] = CF_STATS["f"]  + 1
				return true
			end
		end
	end
	for k, v in pairs(CF_WORDS_EXACT_IGNORE) do
		if msg == k then
			if (type(v) == "boolean" and v == true) or (type(v) == "string" and self["name"] == v) then
				CF_STATS["e"] = CF_STATS["e"] + 1
				return true
			end
		end
	end
	for old, new in pairs(r) do
		newMsg = newMsg:gsub(old, new)
	end
	if newMsg ~= msg then
		CF_STATS["r"] = CF_STATS["r"] + 1
	else
		CF_STATS["p"] = CF_STATS["p"] + 1
	end
	
	return false, newMsg, ...
end

local function addReplace(str)
	old, new = strsplit(CF_DELIM, str)
	old = old or ""
	new = new or ""
	if new and new ~= "" and old and old ~= "" then
		CF_WORDS_REPLACE[old] = new
		generateList()
	end
end

local function delReplace(str)
	if str and str ~= "" then
		CF_WORDS_REPLACE[str] = nil
		generateList()
	end
end

local function addIgnore(str)
	if str and str ~= "" then
		CF_WORDS_IGNORE[str] = true
	end
end

local function delIgnore(str)
	if str and str ~= "" then
		CF_WORDS_IGNORE[str] = nil
	end
end

local function addExactIgnore(str)
	if str and str ~= "" then
		CF_WORDS_EXACT_IGNORE[str] = true
	end
end

local function delExactIgnore(str)
	if str and str ~= "" then
		CF_WORDS_EXACT_IGNORE[str] = nil
	end
end

local function delim(str)
	if str and str ~= "" and str:len() == 1 then
		CF_DELIM = str
	else
		print("Nope")
	end
end

local function info()
	print("CF Commands:")
	print("/cf_list")
	print("/cf_replace <Old>"..CF_DELIM.."<New>")
	print("/cf_del_replace <Old>")
	print("/cf_ignore <Word>")
	print("/cf_del_ignore <Word>")
	print("/cf_exact_ignore <Word>")
	print("/cf_del_exact_ignore <Word>")
	print("/cf_delim <Delimiter>")
	print("/cf_chats")
	print("/cf_chat_add <Chat>")
	print("/cf_chat_del <Chat>")
end

local function list()
	print("CF ignores:")
	for k, v in pairs(CF_WORDS_IGNORE) do
		if type(v) == "bolean" then
			print(k)
		elseif type(v) == "string" then
			print("Tab: " .. v .. ": " .. k)
		end
	end
	print("--------")
	print("CF exact ignores:")
	for k, v in pairs(CF_WORDS_EXACT_IGNORE) do
		if type(v) == "bolean" then
			print(k)
		elseif type(v) == "string" then
			print("Tab: " .. v .. ": " .. k)
		end
	end
	print("--------")
	print("CF replaces:")
	for k, v in pairs(CF_WORDS_REPLACE) do
		print(k.." -> "..v)
	end
end

local function updateWatched()
	for _, chat in pairs(defaultChats) do
		ChatFrame_RemoveMessageEventFilter(chat, filter)
	end
	for _, chat in ipairs(CF_CHATS) do
		ChatFrame_AddMessageEventFilter(chat, filter)
	end
end

local function listWatched()
	print("Watched chats:")
	for _, chat in ipairs(CF_CHATS) do
		print(chat)
	end
end

local function addWatched(chat)
	tinsert(CF_CHATS, chat)
	updateWatched()
end

local function deleteWatched(chat)
	for i, c in ipairs(CF_CHATS) do
		if c == chat then
			tremove(CF_CHATS, i)
		end
	end
	updateWatched()
end

generateList()
updateWatched()

SLASH_ADDREPLACE1 = "/cf_replace"
SlashCmdList["ADDREPLACE"] = addReplace

SLASH_DELREPLACE1 = "/cf_del_replace"
SlashCmdList["DELREPLACE"] = delReplace

SLASH_ADDIGNORE1 = "/cf_ignore"
SlashCmdList["ADDIGNORE"] = addIgnore

SLASH_DELIGNORE1 = "/cf_del_ignore"
SlashCmdList["DELIGNORE"] = delIgnore

SLASH_ADDEXACTIGNORE1 = "/cf_exact_ignore"
SlashCmdList["ADDEXACTIGNORE"] = addExactIgnore

SLASH_DELEXACTIGNORE1 = "/cf_del_exact_ignore"
SlashCmdList["DELEXACTIGNORE"] = delExactIgnore

SLASH_DELIM1 = "/cf_delim"
SlashCmdList["DELIM"] = delim

SLASH_CFINFO1 = "/cf"
SlashCmdList["CFINFO"] = info

SLASH_CFLIST1 = "/cf_list"
SlashCmdList["CFLIST"] = list

SLASH_CFLISTWATCHED1 = "/cf_chats"
SlashCmdList["CFLISTWATCHED"] = listWatched

SLASH_CFADDWATCHED1 = "/cf_chat_add"
SlashCmdList["CFADDWATCHED"] = addWatched

SLASH_CFDELETEWATCHED1 = "/cf_chat_del"
SlashCmdList["CFDELETEWATCHED"] = deleteWatched
